@@ -7,13 +7,7 @@ pyinstaller --noconfirm --onefile --clean --console --collect-all customtkinter
 --add-data "./json_to_gbnf.py:." \
 --add-data "./LICENSE.md:."  \
 --add-data "./MIT_LICENSE_GGML_SDCPP_LLAMACPP_ONLY.md:." \
---add-data "./klite.embd:." \
---add-data "./kcpp_docs.embd:." \
---add-data "./kcpp_sdui.embd:." \
---add-data "./taesd.embd:." \
---add-data "./taesd_xl.embd:." \
---add-data "./taesd_f.embd:." \
---add-data "./taesd_3.embd:." \
+--add-data "./embd_res:./embd_res" \
 --add-data "./koboldcpp_default.so:." \
 --add-data "./koboldcpp_failsafe.so:." \
 --add-data "./koboldcpp_noavx2.so:." \
@@ -22,8 +16,5 @@ pyinstaller --noconfirm --onefile --clean --console --collect-all customtkinter
 --add-data "./koboldcpp_clblast_failsafe.so:." \
 --add-data "./koboldcpp_vulkan_noavx2.so:." \
 --add-data "./koboldcpp_vulkan.so:." \
---add-data "./kokoro_ipa.embd:." \
---add-data "./rwkv_vocab.embd:." \
---add-data "./rwkv_world_vocab.embd:." \
 --version-file "./version.txt" \
 "./koboldcpp.py" -n "koboldcpp"
\ No newline at end of file
